#include "PacmanInterface2D.h"
#include "Color.h"
#include "Hardware2D.h"
#include "LogPtr.h"
#include "String.h"
#include "safe_new.h"

Interface2D * create_game(Hardware2D * hardware)
{
	return new PacmanInterface2D( hardware );
}


PacmanInterface2D::PacmanInterface2D(Hardware2D * hardware)
:	Interface2D( hardware ),
	scene( hardware ),
	pic0( &scene, 0, "data\\help_pic" ),
	pic1( &scene, 1, "data\\intro" ),
	apple( &scene, 2, "data\\apple" ),
	hello( &scene, 3, "data\\small_alphabet" ),
	time_ms( 0 ),
	fps( 2000 )
{
	hardware->set_mode( 240, 320 );

	apple.start( 0, true );
	apple.move( 160, 120 );
	pic0.start( 0, true );
	pic0.move( 0, 0 );
	pic1.start( 0, true );
	pic1.move( 0, 0 );

	hello.move( 0, 0 );
	hello.change_text( "123 Hello, World" );
}

PacmanInterface2D::~PacmanInterface2D()
{
	hardware->stop_mode();
}

bool PacmanInterface2D::render()
{
	bool ok = scene.render( 0, 0, Color(0,0,128) );
	return ok;
}

bool PacmanInterface2D::life_cycle(unsigned delta_time_ms)
{
	fps.record( time_ms );
	LogPtr()->message( String("Fps ") + String::convert( fps.get_fps() ) + "\n" );
	hello.change_text( String("Fps ") + String::convert( int(fps.get_fps()) ) );

	time_ms += delta_time_ms;
	pic1.set_transparency( (time_ms%2550)/10 );

	apple.life_cycle( delta_time_ms );
	pic0.life_cycle( delta_time_ms );
	pic1.life_cycle( delta_time_ms );
	return true;
}
